package com.jeecms.auth.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.jeecms.auth.constants.AuthCommonConstants;
import com.jeecms.auth.constants.AuthCommonConstants.ChannelEnum;
import com.jeecms.auth.constants.AuthCommonConstants.ContentEnum;
import com.jeecms.auth.constants.AuthCommonConstants.SiteEnum;
import com.jeecms.auth.domain.CoreRole;
import com.jeecms.auth.domain.CoreUser;
import com.jeecms.auth.service.CoreRoleService;
import com.jeecms.auth.service.CoreUserService;
import com.jeecms.auth.service.ISiteAuthService;
import com.jeecms.auth.util.AuthUtil;
import com.jeecms.channel.domain.Channel;
import com.jeecms.channel.service.ChannelService;
import com.jeecms.common.base.service.ICacheService;
import com.jeecms.common.constants.ServerModeEnum;
import com.jeecms.common.exception.error.ChannelErrs;
import com.jeecms.common.exception.error.SiteErrs;
import com.jeecms.common.util.CollectionUtil;
import com.jeecms.common.util.GsonUtil;
import com.jeecms.exception.GlobalException;
import com.jeecms.exception.GlobalRuntimeException;
import com.jeecms.system.constants.AuthConstant;
import com.jeecms.system.constants.CacheConstant;
import com.jeecms.system.domain.CmsSite;
import com.jeecms.system.domain.SysAuthData;
import com.jeecms.system.dto.bo.AdminUserBO;
import com.jeecms.system.service.CmsSiteService;
import com.jeecms.system.service.SysAuthDataService;
import com.jeecms.util.SystemContextUtils;
import com.jeecms.utils.GsonUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.jeecms.system.constants.AuthConstant.TOP_ORG_ID;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author tom
 * @date 2022-5-17
 * @copyright 杭州藏茗山科技有限公司 All rights reserved. Notice
 * 仅限于授权后使用，禁止非授权传阅以及私自用于商业目的。
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class SiteAuthServiceImpl implements ISiteAuthService {

    /**
     * 数据权限类型 1.站点
     **/
    public static final int SITE_TYPE = 1;
    /**
     * 数据权限类型  2.栏目
     */
    public static final int CHANNEL_TYPE = 2;
    /**
     * 数据权限类型  3.内容
     */
    public static final int CONTENT_TYPE = 3;

    /***
     * 检验操作是否有权限
     * @param id 站点ID
     * @param operator 操作权限
     * @throw GlobalException
     */
    @Override
    public void checkSiteDataPerm(String name, Integer id, AuthCommonConstants.SiteEnum operator) throws GlobalException {
        /** 开发模式下返回true 忽略权限验证 */
        if (ServerModeEnum.dev.toString().equals(serverMode) || SystemContextUtils.replay()) {
            return;
        }
        AdminUserBO bo = getUser(name);
        if (bo == null) {
            return;
        }
        switch (operator) {
            case VIEW:
                if (!valid(bo, id, SiteEnum.VIEW, null)) {
                    throw new GlobalException(SiteErrs.NO_PERMISSION_VIEW_SITE_ERROR);
                }
                break;
            case NEWCHILD:
                if (!valid(bo, id, SiteEnum.NEWCHILD, null)) {
                    throw new GlobalException(SiteErrs.NO_PERMISSION_NEW_CHILD_SITE_ERROR);
                }
                break;
            case STATIC:
                if (!valid(bo, id, SiteEnum.STATIC, null)) {
                    throw new GlobalException(SiteErrs.NO_PERMISSION_STATIC_SITE_ERROR);
                }
                break;
            case EDIT:
                if (!valid(bo, id, SiteEnum.EDIT, null)) {
                    throw new GlobalException(SiteErrs.NO_PERMISSION_MODIFY_SITE_ERROR);
                }
                break;
            case DELETE:
                if (!valid(bo, id, SiteEnum.DELETE, null)) {
                    throw new GlobalException(SiteErrs.NO_PERMISSION_DEL_SITE_ERROR);
                }
                break;
            case OPE_SITE_OPEN_CLOSE:
                if (!valid(bo, id, SiteEnum.OPE_SITE_OPEN_CLOSE, null)) {
                    throw new GlobalException(SiteErrs.NO_PERMISSION_OPENCLOSE_SITE_ERROR);
                }
                break;
            case PERM_ASSIGN:
                if (!valid(bo, id, SiteEnum.PERM_ASSIGN, null)) {
                    throw new GlobalException(SiteErrs.NO_PERMISSION_PERM_ASSIGN_SITE_ERROR);
                }
                break;
            default:
        }
    }

    /***
     * 检验操作是否有权限
     * @param id 站点ID
     * @param operator 操作权限
     * @throw GlobalException
     */
    @Override
    public boolean getSiteDataPerm(AdminUserBO bo, Integer id, AuthCommonConstants.SiteEnum operator) {
        /** 开发模式下返回true 忽略权限验证 */
        if (ServerModeEnum.dev.toString().equals(serverMode) || SystemContextUtils.replay()) {
            return true;
        }
        if (bo == null) {
            return false;
        }
        return valid(bo, id, operator, null);
    }

    @Override
    public boolean getSiteDataPerm(String userName, Integer siteId, SiteEnum siteEnum) {
        return getSiteDataPerm(getUser(userName), siteId, siteEnum);
    }

    @Override
    public boolean getContentDataPerm(AdminUserBO bo, Integer siteId, Integer id, ContentEnum operator) {
        /** 开发模式下返回true 忽略权限验证 */
        if (ServerModeEnum.dev.toString().equals(serverMode) || SystemContextUtils.replay()) {
            return true;
        }
        if (bo == null) {
            return false;
        }
        return validContent(bo, siteId, id, operator, null, null);
    }

    @Override
    public Map<Integer, Boolean> getContentDataPermBatch(AdminUserBO bo, Integer siteId, Integer id, List<SysAuthData> userAuths, Map<Integer, List<SysAuthData>> roleAuthMap) {
        /** 开发模式下返回true 忽略权限验证 */
        Map<Integer, Boolean> result = new HashMap<>();
        if (ServerModeEnum.dev.toString().equals(serverMode) || SystemContextUtils.replay()) {
            for (ContentEnum ope : ContentEnum.values()) {
                result.put(ope.ordinal(), true);
            }
            return result;
        }
        for (ContentEnum ope : ContentEnum.values()) {
            result.put(ope.ordinal(), false);
        }
        if (bo == null) {
            return result;
        }
        return validContentBatch(bo, siteId, id, userAuths, roleAuthMap);
    }

    @Override
    public void checkContentDataPerm(String name, Integer siteId, Integer id, ContentEnum operator)
            throws GlobalException {
        AdminUserBO bo = getUser(name);
        if (bo == null) {
            return;
        }
        switch (operator) {
            case VIEW:
                if (!validContent(bo, siteId, id, ContentEnum.VIEW, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_VIEW_CHANNEL_CONTENT_ERROR);
                }
                break;
            case NEW:
                if (!validContent(bo, siteId, id, ContentEnum.NEW, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_ADD_CHANNEL_CONTENT_ERROR);
                }
                break;
            case PUBLISH:
                if (!validContent(bo, siteId, id, ContentEnum.PUBLISH, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_PUBLISH_CHANNEL_CONTENT_ERROR);
                }
                break;
            case EDIT:
                if (!validContent(bo, siteId, id, ContentEnum.EDIT, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_EDIT_CHANNEL_CONTENT_ERROR);
                }
                break;
            case FILE:
                if (!validContent(bo, siteId, id, ContentEnum.FILE, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_OFFLINE_CHANNEL_CONTENT_ERROR);
                }
                break;
            case DELETE:
                if (!validContent(bo, siteId, id, ContentEnum.DELETE, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_DELETE_CHANNEL_CONTENT_ERROR);
                }
                break;
            case TOP:
                if (!validContent(bo, siteId, id, ContentEnum.TOP, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_TOP_CHANNEL_CONTENT_ERROR);
                }
                break;
            case MOVE:
                if (!validContent(bo, siteId, id, ContentEnum.MOVE, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_MOVE_CHANNEL_CONTENT_ERROR);
                }
                break;
            case ORDER:
                if (!validContent(bo, siteId, id, ContentEnum.ORDER, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_ORDER_CHANNEL_CONTENT_ERROR);
                }
                break;
            case COPY:
                if (!validContent(bo, siteId, id, ContentEnum.COPY, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_COPY_CHANNEL_CONTENT_ERROR);
                }
                break;
            case QUOTE:
                if (!validContent(bo, siteId, id, ContentEnum.QUOTE, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_QUOTE_CHANNEL_CONTENT_ERROR);
                }
                break;
            case TYPE:
                if (!validContent(bo, siteId, id, ContentEnum.TYPE, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_TYPE_CHANNEL_CONTENT_ERROR);
                }
            case SITE_PUSH:
                if (!validContent(bo, siteId, id, ContentEnum.SITE_PUSH, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_SITE_PUSH_CHANNEL_CONTENT_ERROR);
                }
                break;
            case WEIBO_PUSH:
                if (!validContent(bo, siteId, id, ContentEnum.WEIBO_PUSH, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_WEIBO_PUSH_CHANNEL_CONTENT_ERROR);
                }
                break;
            case WECHAT_PUSH:
                if (!validContent(bo, siteId, id, ContentEnum.WECHAT_PUSH, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_WECHAT_PUSH_CHANNEL_CONTENT_ERROR);
                }
                break;
            case BROWSING_HISTORY:
                if (!validContent(bo, siteId, id, ContentEnum.BROWSING_HISTORY, null, null)) {
                    throw new GlobalException(ChannelErrs.NO_PERMISSION_VISIT_CHANNEL_CONTENT_ERROR);
                }
                break;
            default:
        }
    }

    @Override
    public boolean getChannelDataPerm(AdminUserBO bo, Integer siteId, Integer id, ChannelEnum operator) {
        /** 开发模式下返回true 忽略权限验证 */
        if (ServerModeEnum.dev.toString().equals(serverMode) || SystemContextUtils.replay()) {
            return true;
        }
        if (bo == null) {
            return false;
        }
        return validChannel(bo, siteId, id, operator, null, null);
    }

    @Override
    public Map<Integer, Boolean> getChannelDataPermBatch(AdminUserBO bo, Integer siteId, Integer id, List<SysAuthData> userAuths, Map<Integer, List<SysAuthData>> roleAuthMap) {
        /** 开发模式下返回true 忽略权限验证 */
        Map<Integer, Boolean> result = new HashMap<>();
        if (ServerModeEnum.dev.toString().equals(serverMode) || SystemContextUtils.replay()) {
            for (ChannelEnum ope : ChannelEnum.values()) {
                result.put(ope.ordinal(), true);
            }
            return result;
        }
        for (ChannelEnum ope : ChannelEnum.values()) {
            result.put(ope.ordinal(), false);
        }
        if (bo == null) {
            return result;
        }
        return validChannelBatch(bo, siteId, id, userAuths, roleAuthMap);
    }

    @Override
    public void checkChannelDataPerm(String userName, Integer siteId, Integer channelId, AuthCommonConstants.ChannelEnum channelEnum) {
        AdminUserBO bo = getUser(userName);
        if (bo == null) {
            return;
        }
        switch (channelEnum) {
            case VIEW:
                if (!validChannel(bo, siteId, channelId, ChannelEnum.VIEW, null, null)) {
                    throw new GlobalRuntimeException(ChannelErrs.NO_PERMISSION_VIEW_CHANNEL_ERROR);
                }
                break;
            case NEW:
                if (!validChannel(bo, siteId, channelId, ChannelEnum.NEW, null, null)) {
                    throw new GlobalRuntimeException(ChannelErrs.NO_PERMISSION_NEW_CHANNEL_ERROR);
                }
                break;
            case EDIT:
                if (!validChannel(bo, siteId, channelId, ChannelEnum.EDIT, null, null)) {
                    throw new GlobalRuntimeException(ChannelErrs.NO_PERMISSION_EDIT_CHANNEL_ERROR);
                }
                break;
            case DELETE:
                if (!validChannel(bo, siteId, channelId, ChannelEnum.DELETE, null, null)) {
                    throw new GlobalRuntimeException(ChannelErrs.NO_PERMISSION_DELETE_CHANNEL_ERROR);
                }
                break;
            case STATIC:
                if (!validChannel(bo, siteId, channelId, ChannelEnum.STATIC, null, null)) {
                    throw new GlobalRuntimeException(ChannelErrs.NO_PERMISSION_STATIC_CHANNEL_ERROR);
                }
                break;
            case MERGE:
                if (!validChannel(bo, siteId, channelId, ChannelEnum.MERGE, null, null)) {
                    throw new GlobalRuntimeException(ChannelErrs.NO_PERMISSION_MERGE_CHANNEL_ERROR);
                }
                break;
            case PERMISSION_ASSIGNMENT:
                if (!validChannel(bo, siteId, channelId, ChannelEnum.PERMISSION_ASSIGNMENT, null, null)) {
                    throw new GlobalRuntimeException(ChannelErrs.NO_PERMISSION_PERMISSION_ASSIGNMENT_CHANNEL_ERROR);
                }
                break;
            default:
        }
    }

    /**
     * 计算权限值
     * 先取用户，再取角色，然后合并
     **/
    @Override
    public Integer countForSite(Integer siteId, AdminUserBO bo, Map<Integer, List<SysAuthData>> roleSiteAuthMap) {
        //先假定没有权限
        int sum = 0;
        List<Integer> authList = new ArrayList<>(6);
        int auth;
        if (cacheService.hasKey(CacheConstant.getSiteAuthUserKey(bo.getOrgId(), bo.getUserId(), siteId))) {
            //得到用户权限
            auth = (int) cacheService.get(CacheConstant.getSiteAuthUserKey(bo.getOrgId(), bo.getUserId(), siteId));
        } else {
            //缓存不存在的话，需要重新计算
            auth = countSiteForUser(bo.getOrgId(), bo.getTop(), bo.getOrgIds(), bo.getUserId(), siteId, bo.getSelfAuthSite());
        }
        //先判断用户所属组织是否为顶级组织,如果为顶级组织，则直接取用户权限
        if (bo.getTop()) {
            authList.add(auth);
        } else {
            //汇总所有上级组织权限
            authList.add(auth & countSiteForNotTopOrg(bo.getOrgId(), siteId));
        }
        //得到角色相关权限
        if (!bo.getSelfAuthSite()) {
            for (Map.Entry<Integer, List<Integer>> entry : bo.getRoleOrgIdsForSite().entrySet()) {
                int b;
                //roleAuthDatas null 代表是单项检查，空集合则是未配置
                List<SysAuthData> roleAuthDatas = null;
                if (roleSiteAuthMap != null) {
                    roleAuthDatas = roleSiteAuthMap.get(entry.getKey());
                }
                //缓存不存在的话，需要重新计算
                //如果角色属于顶级组织 entry.getValue()是所属角色的所属组织的上级，如果size=1说明是顶层组织的角色
                //顶层组织的角色直接计算即可，不用与上层组织&运算过滤
                if (entry.getValue().size() == 1) {
                    if (cacheService.hasKey(CacheConstant.getSiteAuthRoleKey(TOP_ORG_ID, entry.getKey(), siteId))) {
                        b = (int) cacheService.get(CacheConstant.getSiteAuthRoleKey(TOP_ORG_ID, entry.getKey(), siteId));
                    } else {
                        b = countSiteForRole(TOP_ORG_ID, true, entry.getKey(), entry.getValue(), siteId, roleAuthDatas);
                    }
                    authList.add(b);
                } else {
                    b = countSiteForRole(null, false, entry.getKey(), entry.getValue(), siteId, roleAuthDatas);
                    authList.add(b);
                }
            }
        }
        /*这里取或运算，只要用户自身有或者所配置的角色有则认定是有权限*/
        for (Integer integer : authList) {
            sum |= integer;
        }
        return sum;
    }

    @Override
    public Integer countForChannel(Integer siteId, Integer channelId, AdminUserBO bo, List<SysAuthData> userAuths, Map<Integer, List<SysAuthData>> roleAuthMap) {
        //先假定没有权限
        int sum = 0;
        List<Integer> authList = new ArrayList<>(6);
        int auth;
        if (cacheService.hasKey(CacheConstant.getChannelAuthUserKey(bo.getOrgId(), bo.getUserId(), siteId, channelId))) {
            //得到用户权限
            auth = (int) cacheService.get(CacheConstant.getChannelAuthUserKey(bo.getOrgId(), bo.getUserId(), siteId, channelId));
        } else {
            //缓存不存在的话，需要重新计算
            auth = countChannelForUser(bo.getOrgId(), bo.getTop(), bo.getOrgIds(), bo.getUserId(),
                    siteId, channelId, AuthConstant.CHANNEL_TYPE, userAuths, bo.getRoleIds());
        }
        //先判断用户所属组织是否为顶级组织,如果为顶级组织，则直接取用户权限
        if (bo.getTop()) {
            authList.add(auth);
        } else {
            //汇总所有上级组织权限
            authList.add(auth & countChannelForNotTopOrg(bo.getOrgId(), siteId, channelId, AuthConstant.CHANNEL_TYPE));
        }
        //得到角色相关权限
        //得到角色相关权限，用户没有单独配置权限则综合角色的，否则不综合角色的权限
        if (!bo.getSelfAuthChannel()) {
            for (Map.Entry<Integer, List<Integer>> entry : bo.getRoleOrgIdsForChannel().entrySet()) {
                int b;
                List<SysAuthData> roleAuthDatas = null;
                if (roleAuthMap != null) {
                    roleAuthDatas = roleAuthMap.get(entry.getKey());
                }
                //如果角色属于顶级组织 entry.getValue()是所属角色的所属组织的上级，如果size=1说明是顶层组织的角色
                //顶层组织的角色直接计算即可，不用与上层组织&运算过滤
                if (entry.getValue().size() == 1) {
                    if (cacheService.hasKey(CacheConstant.getChannelAuthRoleKey(TOP_ORG_ID, entry.getKey(), siteId, channelId))) {
                        b = (int) cacheService.get(CacheConstant.getChannelAuthRoleKey(TOP_ORG_ID, entry.getKey(), siteId, channelId));
                    } else {
                        //缓存不存在的话，需要重新计算
                        b = countChannelForRole(TOP_ORG_ID, true, entry.getKey(), entry.getValue(), siteId, channelId, AuthConstant.CHANNEL_TYPE, roleAuthDatas);
                    }
                    authList.add(b);
                } else {
                    CoreRole role = roleService.findById(entry.getKey());
                    b = countChannelForRole(role.getOrgid(), false, entry.getKey(), entry.getValue(), siteId, channelId, AuthConstant.CHANNEL_TYPE, roleAuthDatas);
                    authList.add(b);
                }
            }
        }
        /*这里取或运算，只要用户自身有或者所配置的角色有则认定是有权限*/
        for (Integer integer : authList) {
            sum |= integer;
        }
        return sum;
    }

    @Override
    public Integer countForContent(Integer siteId, Integer channelId, AdminUserBO bo, List<SysAuthData> userAuths, Map<Integer, List<SysAuthData>> roleAuthMap) {
        //先假定没有权限
        int sum = 0;
        List<Integer> authList = new ArrayList<>(6);
        int auth;
        if (cacheService.hasKey(CacheConstant.getContentAuthUserKey(bo.getOrgId(), bo.getUserId(), siteId, channelId))) {
            //得到用户权限
            auth = (int) cacheService.get(CacheConstant.getContentAuthUserKey(bo.getOrgId(), bo.getUserId(), siteId, channelId));
        } else {
            //缓存不存在的话，需要重新计算
            auth = countChannelForUser(bo.getOrgId(), bo.getTop(), bo.getOrgIds(), bo.getUserId(),
                    siteId, channelId, AuthConstant.CONTENT_TYPE, userAuths, bo.getRoleIds());
        }
        //先判断用户所属组织是否为顶级组织,如果为顶级组织，则直接取用户权限
        if (bo.getTop()) {
            authList.add(auth);
        } else {
            //汇总所有上级组织权限
            authList.add(auth & countChannelForNotTopOrg(bo.getOrgId(), siteId, channelId, AuthConstant.CONTENT_TYPE));
        }
        //得到角色相关权限，用户没有单独配置权限则综合角色的，否则不综合角色的权限
        if (!bo.getSelfAuthContent()) {
            List<SysAuthData> roleAuths = null;
            for (Map.Entry<Integer, List<Integer>> entry : bo.getRoleOrgIdsForContent().entrySet()) {
                if (roleAuthMap != null) {
                    roleAuths = roleAuthMap.get(entry.getKey());
                }
                int b;
                //如果角色属于顶级组织 entry.getValue()是所属角色的所属组织的上级，如果size=1说明是顶层组织的角色
                //顶层组织的角色直接计算即可，不用与上层组织&运算过滤
                if (entry.getValue().size() == 1) {
                    if (cacheService.hasKey(CacheConstant.getContentAuthRoleKey(TOP_ORG_ID, entry.getKey(), siteId, channelId))) {
                        b = (int) cacheService.get(CacheConstant.getContentAuthRoleKey(TOP_ORG_ID, entry.getKey(), siteId, channelId));
                    } else {
                        //缓存不存在的话，需要重新计算
                        b = countChannelForRole(TOP_ORG_ID, true, entry.getKey(), entry.getValue(), siteId, channelId, AuthConstant.CONTENT_TYPE, roleAuths);
                    }
                    authList.add(b);
                } else {
                    CoreRole role = roleService.findById(entry.getKey());
                    b = countChannelForRole(role.getOrgid(), false, entry.getKey(), entry.getValue(), siteId, channelId, AuthConstant.CONTENT_TYPE, roleAuths);
                    authList.add(b);
                }
            }
        }
        /*这里取或运算，只要用户自身有或者所配置的角色有则认定是有权限*/
        for (Integer integer : authList) {
            sum |= integer;
        }
        return sum;
    }

    @Override
    public AdminUserBO getUser(String username) {
        Object boString = cacheService.get(CacheConstant.getAdminUserBO(username));
        if (boString != null) {
            return GsonUtils.gsonToBean(boString.toString(), AdminUserBO.class);
        }
        return authDataService.saveUserBo(username);
    }

    @Override
    public List<CmsSite> getViewSites(String username) {
        return getSitesByOpe(username, SiteEnum.VIEW);
    }

    @Override
    public List<CmsSite> getSitesByOpe(String username, SiteEnum siteEnum) {
        List<CmsSite> viewSites = CollUtil.newArrayList();
        AdminUserBO userBO = getUser(username);
        if (userBO == null) {
            return viewSites;
        }
        List<CmsSite> sites = siteService.findAll(false, true);
        CoreUser user = userService.findByUsername(username);
        Map<Integer, List<SysAuthData>> roleSiteAuthMap = new HashMap<>();
        //避免站点多批量查询，角色又没有数据查询缓存没有命中导致无效
        for (Integer roleId : user.getRoleIds()) {
            List<SysAuthData> auths = authDataService.findAuths(null, roleId, null, AuthConstant.SITE_TYPE);
            roleSiteAuthMap.put(roleId, auths);
        }
        for (CmsSite s : sites) {
            if (valid(userBO, s.getId(), siteEnum, roleSiteAuthMap)) {
                viewSites.add(s);
            }
        }
        return viewSites;
    }

    /**
     * 校验站点权限
     * true有权限，false无权限
     **/
    private boolean valid(AdminUserBO bo, Integer id, SiteEnum siteEnum, Map<Integer, List<SysAuthData>> roleSiteAuthMap) {
        Integer a = countForSite(id, bo, roleSiteAuthMap);
        String s = AuthUtil.toBinary(a, AuthCommonConstants.SITE_DIGITS);
        char[] chars = s.toCharArray();
        return Character.getNumericValue(chars[siteEnum.ordinal()]) == 1;
    }

    /**
     * 校验内容权限
     * true有权限，false无权限
     **/
    private boolean validContent(AdminUserBO bo, Integer siteId, Integer id, ContentEnum contentEnum, List<SysAuthData> userAuths,
                                 Map<Integer, List<SysAuthData>> roleAuthDatas) {
        Integer a = countForContent(siteId, id, bo, userAuths, roleAuthDatas);
        String s = AuthUtil.toBinary(a, AuthCommonConstants.CONTENT_DIGITS);
        char[] chars = s.toCharArray();
        return Character.getNumericValue(chars[contentEnum.ordinal()]) == 1;
    }


    /**
     * 校验栏目权限
     *
     * @param bo          用户权限
     * @param siteId      站点id
     * @param id          栏目id
     * @param channelEnum 操作类型
     * @param userAuths   用户配置的权限数据，批量查询这里传入，单条数据权限传入null，可不传递
     * @param roleAuthMap
     * @return true有权限，false无权限
     */
    private boolean validChannel(AdminUserBO bo, Integer siteId, Integer id, ChannelEnum channelEnum, List<SysAuthData> userAuths, Map<Integer, List<SysAuthData>> roleAuthMap) {
        Integer a = countForChannel(siteId, id, bo, userAuths, roleAuthMap);
        String s = AuthUtil.toBinary(a, AuthCommonConstants.CHANNEL_PERMISSION_DIGITS);
        char[] chars = s.toCharArray();
        return Character.getNumericValue(chars[channelEnum.ordinal()]) == 1;
    }

    /**
     * 校验栏目权限
     * map key是栏目操作类型枚举位置   true有权限，false无权限
     **/
    private Map<Integer, Boolean> validChannelBatch(AdminUserBO bo, Integer siteId, Integer id, List<SysAuthData> userAuths, Map<Integer, List<SysAuthData>> roleAuthMap) {
        Integer a = countForChannel(siteId, id, bo, userAuths, roleAuthMap);
        String s = AuthUtil.toBinary(a, AuthCommonConstants.CHANNEL_PERMISSION_DIGITS);
        char[] chars = s.toCharArray();
        Map<Integer, Boolean> result = new HashMap<>();
        for (ChannelEnum ope : ChannelEnum.values()) {
            result.put(ope.ordinal(), Character.getNumericValue(chars[ope.ordinal()]) == 1);
        }
        return result;
    }

    public boolean validChannel(Integer channelId, ChannelEnum channelEnum,
            Map<Integer, Integer> channelAuthMap) {
        Integer auth = channelAuthMap.get(channelId);
        String s = AuthUtil.toBinary(auth, AuthCommonConstants.CHANNEL_PERMISSION_DIGITS);
        char[] chars = s.toCharArray();
        return Character.getNumericValue(chars[channelEnum.ordinal()]) == 1;
    }

    /**
     * 校验栏目权限
     * map key是栏目操作类型枚举位置   true有权限，false无权限
     **/
    private Map<Integer, Boolean> validContentBatch(AdminUserBO bo, Integer siteId, Integer id,
            List<SysAuthData> userAuths, Map<Integer, List<SysAuthData>> roleAuthMap) {
        Integer a = countForContent(siteId, id, bo, userAuths, roleAuthMap);
        String s = AuthUtil.toBinary(a, AuthCommonConstants.CONTENT_DIGITS);
        char[] chars = s.toCharArray();
        Map<Integer, Boolean> result = new HashMap<>();
        for (ContentEnum ope : ContentEnum.values()) {
            result.put(ope.ordinal(), Character.getNumericValue(chars[ope.ordinal()]) == 1);
        }
        return result;
    }

    /**
     * 非顶层组织直接计算组织 站点类权限值
     *
     * @param orgId  组织id
     * @param siteId 站点ID
     * @return
     */
    private Integer countSiteForNotTopOrg(Integer orgId, Integer siteId) {
        SysAuthData auth = authDataService.findAuth(orgId, null, null, siteId, AuthConstant.SITE_TYPE);
        if (auth != null) {
            cacheService.set(CacheConstant.getSiteAuthOrgKey(orgId, siteId), auth.getDataAuth());
            return auth.getDataAuth();
        }
        cacheService.set(CacheConstant.getSiteAuthOrgKey(orgId, siteId), AuthConstant.SITE_ALL_SELECT);
        //没有找到则继承 认定为有权限
        return AuthConstant.SITE_ALL_SELECT;
    }

    /**
     * 非顶层组织直接计算组织 栏目类权限值
     *
     * @param orgId    组织id
     * @param siteId   站点ID
     * @param dataType 数据类型，栏目和内容类型
     * @return
     */
    private Integer countChannelForNotTopOrg(Integer orgId, Integer siteId, Integer channelId, int dataType) {
        int allSelectPerm = AuthConstant.CHANNEL_ALL_SELECT;
        if (AuthConstant.CONTENT_TYPE == dataType) {
            allSelectPerm = AuthConstant.CONTENT_ALL_SELECT;
        }
        //避免重复查询，这里只是用用户 类型查询数据库
        String authCountKey = CacheConstant.getAuthCountOrgKey(orgId, dataType);
        int orgAuthCount;
        if (cacheService.hasKey(authCountKey)) {
            orgAuthCount = (Integer) cacheService.get(authCountKey);
        } else {
            orgAuthCount = authDataService.countByOrgIdAndDataType(orgId, dataType);
            cacheService.set(authCountKey, orgAuthCount);
        }
        SysAuthData auth = null;
        if (orgAuthCount > 0) {
            auth = authDataService.findAuth(orgId, null, null, channelId, dataType);
        }
        if (auth != null) {
            allSelectPerm = auth.getDataAuth();
        } else {
            //有单独分配权限并且没有该 权限值 则没有权限
            if (orgAuthCount > 0) {
                allSelectPerm = 0;
            }
        }
        if (AuthConstant.CONTENT_TYPE == dataType) {
            cacheService.set(CacheConstant.getContentAuthOrgKey(orgId, siteId, channelId), allSelectPerm);
        } else {
            cacheService.set(CacheConstant.getChannelAuthOrgKey(orgId, siteId, channelId), allSelectPerm);
        }
        //没有找到则继承 认定为有权限
        return allSelectPerm;
    }


    /**
     * 计算组织站点类权限值
     *
     * @param orgId    组织id
     * @param isTopOrg 是否是顶层组织
     * @param orgIds   非顶层的上级id
     * @param siteId   站点id
     * @return
     */
    public Integer countSiteForOrg(Integer orgId, boolean isTopOrg, List<Integer> orgIds, Integer siteId) {
        if (isTopOrg) {
            return AuthConstant.SITE_ALL_SELECT;
        } else {
            int orgSum = AuthConstant.SITE_ALL_SELECT;
            List<Integer> orgAuths = new ArrayList<>(orgIds.size());
            //得到上级全部组织权限
            for (Integer userOrgId : orgIds) {
                //忽略组织为1数据
                if (userOrgId.equals(TOP_ORG_ID)) {
                    continue;
                }
                int orgAuth;
                if (cacheService.hasKey(CacheConstant.getSiteAuthOrgKey(userOrgId, siteId))) {
                    //得到用户组织权限
                    orgAuth = (int) cacheService.get(CacheConstant.getSiteAuthOrgKey(userOrgId, siteId));
                } else {
                    //缓存不存在的话，需要重新计算
                    orgAuth = countSiteForNotTopOrg(orgId, siteId);
                }
                orgAuths.add(orgAuth);
            }
            //进行与运算只要一个没有权限则是没权限
            for (Integer orgAu : orgAuths) {
                orgSum &= orgAu;
            }
            return orgSum;
        }
    }

    /**
     * 计算组织栏目类权限值
     *
     * @param orgId    组织id
     * @param isTopOrg 是否是顶层组织
     * @param orgIds   非顶层的上级id
     * @param siteId   站点id
     * @param dataType
     * @return
     */
    public Integer countChannelForOrg(Integer orgId, boolean isTopOrg, List<Integer> orgIds, Integer siteId, Integer channelId, int dataType) {
        if (isTopOrg) {
            return dataType == AuthConstant.CHANNEL_TYPE ? AuthConstant.CHANNEL_ALL_SELECT : AuthConstant.CONTENT_ALL_SELECT;
        } else {
            int orgSum = dataType == AuthConstant.CHANNEL_TYPE ? AuthConstant.CHANNEL_ALL_SELECT : AuthConstant.CONTENT_ALL_SELECT;
            List<Integer> orgAuths = new ArrayList<>(orgIds.size());
            //得到上级全部组织权限
            for (Integer userOrgId : orgIds) {
                //忽略组织为1数据
                if (userOrgId.equals(TOP_ORG_ID)) {
                    continue;
                }
                int orgAuth;
                String cacheKey = dataType == AuthConstant.CHANNEL_TYPE ? CacheConstant.getChannelAuthOrgKey(userOrgId, siteId, channelId) :
                        CacheConstant.getContentAuthOrgKey(userOrgId, siteId, channelId);
                if (cacheService.hasKey(cacheKey)) {
                    //得到用户组织权限
                    orgAuth = (int) cacheService.get(cacheKey);
                } else {
                    //缓存不存在的话，需要重新计算
                    orgAuth = countChannelForNotTopOrg(orgId, siteId, channelId, dataType);
                }
                orgAuths.add(orgAuth);
            }
            //进行与运算只要一个没有权限则是没权限
            for (Integer orgAu : orgAuths) {
                orgSum &= orgAu;
            }
            return orgSum;
        }
    }

    public Integer countSiteForRole(Integer orgId, boolean orgIsTop, Integer roleId, List<Integer> roleOrgIds, Integer siteId, List<SysAuthData> roleSiteAuths) {
        int roleAuth = AuthConstant.SITE_ALL_SELECT;
        //roleAuthDatas null 代表是单项检查，则再次查询一次
        //roleAuthDatas 空集合则是未配置继承组织，如果角色权限不是空集合又没有这项配置则说明没有权限
        if (roleSiteAuths == null) {
            roleSiteAuths = authDataService.findAuths(null, roleId, null, AuthConstant.SITE_TYPE);
        }
        if (orgIsTop) {
            SysAuthData auth = authDataService.findAuth(null, roleId, null, siteId, AuthConstant.SITE_TYPE);
            int authPerm = AuthConstant.SITE_ALL_SELECT;
            if (auth != null) {
                authPerm = auth.getDataAuth();
            } else if (!roleSiteAuths.isEmpty()) {
                authPerm = 0;
            }
            cacheService.set(CacheConstant.getSiteAuthRoleKey(orgId, roleId, siteId), authPerm);
            return authPerm;
        } else {
            List<Integer> roleAuths = new ArrayList<>(roleOrgIds.size());
            //得到角色的全部权限
            for (Integer roleOrgId : roleOrgIds) {
                int a1;
                if (cacheService.hasKey(CacheConstant.getSiteAuthRoleKey(roleOrgId, roleId, siteId))) {
                    //得到用户组织权限
                    a1 = (int) cacheService.get(CacheConstant.getSiteAuthRoleKey(roleOrgId, roleId, siteId));
                } else {
                    //缓存不存在的话，需要重新计算
                    SysAuthData auth = authDataService.findAuth(null, roleId, null, siteId, AuthConstant.SITE_TYPE);
                    int authPerm;
                    //配置了读取自身，否则继承读取组织的权限
                    if (auth != null) {
                        authPerm = auth.getDataAuth();
                    } else {
                        authPerm = countSiteForNotTopOrg(roleOrgId, siteId);
                    }
                    cacheService.set(CacheConstant.getSiteAuthRoleKey(roleOrgId, roleId, siteId), authPerm);
                    return authPerm;
                }
                roleAuths.add(a1);
            }
            //进行与运算
            for (Integer au : roleAuths) {
                roleAuth &= au;
            }
        }
        return roleAuth;
    }

    public Integer countSiteForUser(Integer orgId, boolean isTopOrg, List<Integer> userOrgIds, Integer userId, Integer siteId, boolean selfAuthSite) {
        int orgAuth = countSiteForOrg(orgId, isTopOrg, userOrgIds, siteId);
        SysAuthData userAuth = authDataService.findAuth(null, null, userId, siteId, AuthConstant.SITE_TYPE);
        if (userAuth != null) {
            int authPerm = orgAuth & userAuth.getDataAuth();
            cacheService.set(CacheConstant.getSiteAuthUserKey(orgId, userId, siteId), authPerm);
            userAuth.setDataAuth(authPerm);
            try {
                authDataService.update(userAuth);
            } catch (com.jeecms.common.exception.GlobalException e) {
                log.error(e.getMessage());
            }
            return userAuth.getDataAuth();
        } else {
            //如果单独分配来了权限，但是自身站点没有配置则没有权限
            if (selfAuthSite) {
                orgAuth = 0;
            }
        }
        cacheService.set(CacheConstant.getSiteAuthUserKey(orgId, userId, siteId), orgAuth);
        //没有找到则继承 认定为有权限
        return orgAuth;
    }

    public Integer countChannelForRole(Integer orgId, boolean orgIsTop, Integer roleId, List<Integer> roleOrgIds,
                                       Integer siteId, Integer channelId, int dataType, List<SysAuthData> auths) {
        int roleAuth = AuthConstant.CHANNEL_ALL_SELECT;
        if (AuthConstant.CONTENT_TYPE == dataType) {
            roleAuth = AuthConstant.CONTENT_ALL_SELECT;
        }
        String cacheKey = CacheConstant.getChannelAuthRoleKey(orgId, roleId, siteId, channelId);
        if (AuthConstant.CONTENT_TYPE == dataType) {
            cacheKey = CacheConstant.getContentAuthRoleKey(orgId, roleId, siteId, channelId);
        }
        if (cacheService.hasKey(cacheKey)) {
            return (Integer) cacheService.get(cacheKey);
        }
        boolean hasAssign = false;
//        if (orgIsTop) {
        //避免重复查询，这里只是用用户 类型查询数据库,可以外部调用避免循环的时候重复查询  null是单条数据权限验证，空集合也不走查询库了
        SysAuthData auth = null;
        if (auths == null) {
            hasAssign = authDataService.countByRoleIdAndDataType(roleId, dataType) > 0;
            if (hasAssign) {
                auth = authDataService.findAuth(null, roleId, null, channelId, dataType);
            }
        } else {
            hasAssign = !auths.isEmpty();
            List<SysAuthData> filters = auths.stream().filter(a -> a.getDataId().equals(channelId) && a.getDataType().equals(dataType)).collect(Collectors.toList());
            if (!filters.isEmpty()) {
                auth = filters.get(0);
            }
        }

        //有独立配置权限取，否则如果有单独配置权限，又没有该项则没有权限
        int authPerm = roleAuth;
        if (auth != null) {
            authPerm = auth.getDataAuth();
        } else {
            //有独立分配则又没有则没有权限，否则继承
            if (hasAssign) {
                authPerm = 0;
            } else {
                authPerm = countChannelForOrg(orgId, orgIsTop, roleOrgIds, siteId, channelId, dataType);
            }
        }
        if (AuthConstant.CONTENT_TYPE == dataType) {
            cacheService.set(cacheKey, authPerm);
        } else {
            cacheService.set(cacheKey, authPerm);
        }
        return authPerm;
//        } else {
//            List<Integer> roleAuths = new ArrayList<>(roleOrgIds.size());
//            //得到角色的全部权限
//            for (Integer roleOrgId : roleOrgIds) {
//                int a1;
//                String roleCacheKey = AuthConstant.CONTENT_TYPE == dataType ? CacheConstant.getContentAuthRoleKey(orgId, roleId, siteId, channelId) :
//                        CacheConstant.getChannelAuthRoleKey(orgId, roleId, siteId, channelId);
//                if (cacheService.hasKey(roleCacheKey)) {
//                    //得到用户组织权限
//                    a1 = (int) cacheService.get(roleCacheKey);
//                } else {
//                    //缓存不存在的话，需要重新计算
//                    SysAuthData auth = authDataService.findAuth(null, roleId, null, channelId, dataType);
//                    int authPerm;
//                    //配置了读取自身，否则 如果角色独立配置了 则认为没有权限，否则继承读取组织的权限
//                    if (auth != null) {
//                        authPerm = auth.getDataAuth();
//                    } else {
//                        hasAssign = authDataService.countByRoleIdAndDataType(roleId, dataType) > 0;
//                        if (hasAssign) {
//                            authPerm = 0;
//                        } else {
//                            authPerm = countChannelForNotTopOrg(roleOrgId, siteId, channelId, dataType);
//                        }
//                    }
//                    if (AuthConstant.CONTENT_TYPE == dataType) {
//                        cacheService.set(cacheKey, authPerm);
//                    } else {
//                        cacheService.set(cacheKey, authPerm);
//                    }
//                    return authPerm;
//                }
        //                roleAuths.add(a1);
        //            }
        //            //进行与运算
        //            for (Integer au : roleAuths) {
        //                roleAuth &= au;
        //            }
        //        }
        //        return roleAuth;
    }

    public Integer countChannelForUser(Integer orgId, boolean isTopOrg, List<Integer> userOrgIds,
            Integer userId, Integer siteId, Integer channelId, int dataType,
            List<SysAuthData> userAuths, List<Integer> roleIds) {

        //        SysAuthData auth = authDataService.findAuth(null, null, userId, channelId, dataType);
        //避免重复查询，这里只是用用户 类型查询数据库
        //这里选择可以从外传递是为了循环栏目取权限的时候可以这里查询只查询一次，避免非必要查询
        String cacheKey = CacheConstant.getContentAuthUserKey(orgId, userId, siteId, channelId);
        if (AuthConstant.CHANNEL_TYPE == dataType) {
            cacheKey = CacheConstant.getChannelAuthUserKey(orgId, userId, siteId, channelId);
        }
        if (cacheService.hasKey(cacheKey)) {
            return (Integer) cacheService.get(cacheKey);
        }
        int orgAuth = countChannelForOrg(orgId, isTopOrg, userOrgIds, siteId, channelId, dataType);
        if (userAuths == null) {
            userAuths = authDataService.findAuths(null, null, userId, dataType);
        }
        SysAuthData auth = null;
        if (!userAuths.isEmpty()) {
            List<SysAuthData> filters = userAuths.stream().filter(a -> a.getDataId().equals(channelId)).collect(Collectors.toList());
            if (!filters.isEmpty()) {
                auth = filters.get(0);
            }
        }
        //用户单独配置了以自身为准
        if (!userAuths.isEmpty()) {
            //用户配置项里面未包含栏目则0没有权限
            if (auth == null) {
                return 0;
            }
            int authPerm = orgAuth & auth.getDataAuth();
            if (AuthConstant.CHANNEL_TYPE == dataType) {
                cacheService.set(cacheKey, authPerm);
            } else {
                cacheService.set(cacheKey, authPerm);
            }
            auth.setDataAuth(authPerm);
            try {
                authDataService.update(auth);
            } catch (com.jeecms.common.exception.GlobalException e) {
                log.error(e.getMessage());
            }
            return auth.getDataAuth();
        } else {
            /**用户没有配置权限，且用户有角色 这里给0则去综合角色的权限*/
            if (CollectionUtil.isNotEmpty(roleIds)) {
                return 0;
            }
            //否则继承组织
            if (AuthConstant.CHANNEL_TYPE == dataType) {
                cacheService.set(cacheKey, orgAuth);
            } else {
                cacheService.set(cacheKey, orgAuth);
            }
        }
        return orgAuth;
    }

    @Override
    public List<Channel> getViewChannels(String username, Integer siteId) {
        //查询是否存在用户缓存
        AdminUserBO bo = getUser(username);
        Map<Integer, Integer> map;
        String cacheKey = CacheConstant.getChannelAuthForUserForSite(bo.getOrgId(), bo.getUserId(), siteId);
        if (cacheService.hasKey(cacheKey)) {
            Object o = cacheService.get(cacheKey);
            map = GsonUtil.gsonToMaps(o.toString());
        } else {
            //没有去查询
            map = authDataService.getSiteChannelAuth(username, CHANNEL_TYPE, siteId);
            cacheService.set(cacheKey, GsonUtils.gsonString(map));
        }
        map.entrySet().removeIf(entry -> entry.getValue() == 0);
        //得到有权限的栏目
        List<Channel> channels = CollUtil.newArrayList();
        for (Integer cid : map.keySet()) {
            Channel c = channelService.findById(cid);
            channels.add(c);
        }
        return channels.stream().filter(x -> x != null && x.getSiteId().equals(siteId)).collect(Collectors.toList());
//        return getChannelsByOpe(username,siteId,ChannelEnum.VIEW);
    }

    @Override
    public List<Channel> getViewChannels(String username, Integer siteId, Boolean lazy, Integer parentId) {
        List<Channel> channelList;
        if (lazy) {
            //parentId null查询顶层
            if (parentId == null) {
                parentId = 0;
            }
            channelList = channelService.findListByParentId(siteId, parentId, true, true);
        } else {
            channelList = channelService.getList(siteId, false);
        }
        AdminUserBO userPermBo = getUser(username);
        List<Channel> filterChannels = CollUtil.newArrayList();
        for (Channel c : channelList) {
            boolean canView = validChannel(userPermBo, siteId, c.getId(), ChannelEnum.VIEW, null, null);
            if (canView) {
                filterChannels.add(c);
            }
        }
        return filterChannels;
    }

    @Override
    public List<Channel> getContentChannelsByOpe(String username, Integer siteId, ContentEnum contentEnum, Boolean lazy, Integer parentId) {
        List<Channel> channelList;
        if (lazy) {
            //parentId null查询顶层
            if (parentId == null) {
                parentId = 0;
            }
            channelList = channelService.findListByParentId(siteId, parentId, true, true);
        } else {
            channelList = channelService.getList(siteId, false);
        }
        //非懒加载主要再调用新建内容的上级栏目选择那，这个时候使用上缓存
        CoreUser user = userService.findByUsername(username);
        String cacheKey = CacheConstant.getSiteContentAuthWithPermForUser(user.getOrgId(), user.getId(), siteId, contentEnum.ordinal());
        if (!lazy && cacheService.hasKey(cacheKey)) {
            String cacheObj = (String) cacheService.get(cacheKey);
            if (cacheObj != null) {
                List<Integer> channelIds = GsonUtils.gsonToList(cacheObj, Integer.class);
                List<Channel> channels = CollectionUtil.newArrayList();
                for (Integer cId : channelIds) {
                    channels.add(channelService.findById(cId));
                }
                return channels;
            }
        }
        AdminUserBO userPermBo = getUser(username);
        List<Channel> filterChannels = CollUtil.newArrayList();
        List<SysAuthData> userAuthDatas = authDataService.findAuths(null, null, userPermBo.getUserId(), AuthConstant.CONTENT_TYPE);
        Map<Integer, List<SysAuthData>> roleAuthMap = new HashMap<>();
        //用户未配置则需要取角色的
        if (userAuthDatas.isEmpty()) {
            for (Integer roleId : user.getRoleIds()) {
                List<SysAuthData> roleAuthDatas = authDataService.findAuths(null, roleId, null, AuthConstant.CONTENT_TYPE);
                roleAuthMap.put(roleId, roleAuthDatas);
            }
        }
        for (Channel c : channelList) {
            boolean canView = validContent(userPermBo, siteId, c.getId(), contentEnum, userAuthDatas, roleAuthMap);
            if (canView) {
                filterChannels.add(c);
            }
        }
        if (!lazy) {
            cacheService.set(cacheKey, GsonUtils.gsonString(Channel.fetchIds(filterChannels)));
        }
        return filterChannels;
    }

    @Override
    public List<Channel> getChannelsByOpe(String username, Integer siteId, ChannelEnum channelEnum) {
        AdminUserBO bo = getUser(username);
        if (bo == null) {
            return CollUtil.newArrayList();
        }
        //得到有权限的栏目
        AdminUserBO userBO = getUser(username);
        List<Channel> hasPermChannels = CollUtil.newArrayList();
        List<Channel> channels = channelService.getChanelByRecycle(siteId, false);
        List<SysAuthData> userAuthDatas = authDataService.findAuths(null, null, bo.getUserId(), AuthConstant.CHANNEL_TYPE);
        Map<Integer, List<SysAuthData>> roleAuthMap = new HashMap<>();
        //用户未配置则需取角色
        if (userAuthDatas.isEmpty() && userBO != null && userBO.getRoleIds() != null) {
            for (Integer roleId : userBO.getRoleIds()) {
                roleAuthMap.put(roleId, authDataService.findAuths(null, roleId, null, AuthConstant.CHANNEL_TYPE));
            }
        }
        for (Channel s : channels) {
            if (validChannel(userBO, siteId, s.getId(), channelEnum, userAuthDatas, roleAuthMap)) {
                hasPermChannels.add(s);
            }
        }
        hasPermChannels = hasPermChannels.stream().filter(c -> c != null && !c.getRecycle() && !c.getHasDeleted()).collect(Collectors.toList());
        return hasPermChannels;
    }

    @Override
    public List<Channel> getViewContentChannels(String username, Integer siteId) {
        return getContentChannelsByOpe(username, siteId, ContentEnum.VIEW);
    }

    @Override
    public List<Channel> getContentChannelsByOpe(String username, Integer siteId, ContentEnum contentEnum) {
        AdminUserBO bo = getUser(username);
        if (bo == null) {
            return CollUtil.newArrayList();
        }
//        String cacheKey = CacheConstant.getSiteContentChanneIdWithPermForUser(bo.getOrgId(), bo.getUserId(), siteId, contentEnum.ordinal());
//        if (cacheService.hasKey(cacheKey)) {
//            String jsonStr = (String) cacheService.get(cacheKey);
//            List<Integer> channelIds = GsonUtils.gsonToList(jsonStr, Integer.class);
//            List<Channel> channels = CollUtil.newArrayList();
//            for (Integer channelId : channelIds) {
//                channels.add(channelService.findById(channelId));
//            }
//            return channels;
//        }
//        //得到有权限的栏目
//        AdminUserBO userBO = getUser(username);
//        List<Channel> hasPermChannels = CollUtil.newArrayList();
//        List<Channel> channels = channelService.getChanelByRecycle(siteId, false);
//        List<SysAuthData> userAuthDatas = authDataService.findAuths(null, null, bo.getUserId(), AuthConstant.CONTENT_TYPE);
//        Map<Integer, List<SysAuthData>> roleAuthMap = new HashMap<>();
//        CoreUser user = userService.findByUsername(username);
//        if (userAuthDatas.isEmpty()) {
//            for (Integer roleId : user.getRoleIds()) {
//                List<SysAuthData> roleAuthDatas = authDataService.findAuths(null, roleId, null, AuthConstant.CONTENT_TYPE);
//                roleAuthMap.put(roleId, roleAuthDatas);
//            }
//        }
//        for (Channel s : channels) {
//            if (validContent(userBO, siteId, s.getId(), contentEnum, userAuthDatas, roleAuthMap)) {
//                hasPermChannels.add(s);
//            }
//        }
//        hasPermChannels = hasPermChannels.stream().filter(c -> c != null && !c.getRecycle() && !c.getHasDeleted()).collect(Collectors.toList());
//        cacheService.set(cacheKey, GsonUtils.gsonString(CollectionUtil.convert(hasPermChannels, Channel::getId)));
//        return hasPermChannels;
        Map<Integer, Integer> map;
        String cacheKey = CacheConstant.getContentAuthForUserForSite(bo.getOrgId(), bo.getUserId(), siteId);
        if (cacheService.hasKey(cacheKey)) {
            Object o = cacheService.get(cacheKey);
            map = GsonUtil.gsonToMaps(o.toString());
        } else {
            //没有去查询
            map = authDataService.getSiteChannelAuth(username, CONTENT_TYPE, siteId);
            cacheService.set(cacheKey, GsonUtils.gsonString(map));
        }
        //将没有权限的数据过滤
        map.entrySet().removeIf(entry -> entry.getValue() < AuthConstant.getContentEnumMin(contentEnum));
        //得到有权限的栏目
        List<Channel> channels = CollUtil.newArrayList();
        for (Integer cid : map.keySet()) {
            Channel c = channelService.findById(cid);
            channels.add(c);
        }
        return channels.stream().filter(x -> x != null && x.getSiteId().equals(siteId)).collect(Collectors.toList());
    }

    private static final Logger log = LoggerFactory.getLogger(SiteAuthServiceImpl.class);

    @Value("${spring.profiles.active}")
    private String serverMode;
    @Autowired
    private ICacheService cacheService;
    @Autowired
    private SysAuthDataService authDataService;
    @Autowired
    private CmsSiteService siteService;
    @Autowired
    private CoreRoleService roleService;
    @Autowired
    private ChannelService channelService;
    @Autowired
    private CoreUserService userService;
}
